@@ -1,8 +1,8 @@
 # $Author: ddumont $
-# $Date: 2009-06-22 14:02:02 +0200 (lun 22 jun 2009) $
-# $Revision: 978 $
+# $Date: 2010-02-18 18:29:48 +0100 (Thu, 18 Feb 2010) $
+# $Revision: 1086 $
 
-#    Copyright (c) 2005-2007 Dominique Dumont.
+#    Copyright (c) 2005-2010 Dominique Dumont.
 #
 #    This file is part of Config-Model.
 #
@@ -22,17 +22,17 @@
 
 package Config::Model::CheckList ;
 use Config::Model::Exception ;
-use Scalar::Util qw(weaken) ;
 use warnings ;
 use Carp;
 use strict;
-
-
+use Log::Log4perl qw(get_logger :levels);
 
 use base qw/Config::Model::WarpedThing/ ;
 
 use vars qw($VERSION) ;
-$VERSION = sprintf "1.%04d", q$Revision: 978 $ =~ /(\d+)/;
+$VERSION = sprintf "1.%04d", q$Revision: 1086 $ =~ /(\d+)/;
+
+my $logger = get_logger("Tree::Element::CheckList") ;
 
 =head1 NAME
 
@@ -132,6 +132,7 @@ sub new {
 
     $self->cl_init ;
 
+    $logger->info("Created check_list element $self->{element_name}");
     return $self ;
 }
 
@@ -189,6 +190,10 @@ details. (optional)
 List ref to specify the check list items which are "on" by default.
 (optional)
 
+=item ordered
+
+Specify whether the order of checked items must be preserved. 
+
 =item help
 
 Hash ref to provide informations on the check list items.
@@ -273,7 +278,13 @@ sub set_properties {
     my $self = shift ;
 
     # cleanup all parameters that are handled by warp
-    map(delete $self->{$_}, @allowed_warp_params) ;
+    map(delete $self->{$_}, @allowed_warp_params ) ;
+
+    if ($logger->is_debug()) {
+      my %h = @_;
+      my $keys = join(',',keys %h) ;
+      $logger->debug("set_properties called on $self->{element_name} with $keys");
+    }
 
     # merge data passed to the constructor with data passed to set
     my %args = (%{$self->{backup}},@_ );
@@ -295,22 +306,19 @@ sub set_properties {
 
     if (defined $args{default_list}) {
 	$self->{default_list} = delete $args{default_list} ;
-	my %h = map { $_ => 1 } @{$self->{default_list}} ;
-	$self->{default_data} = \%h ;
-    }
-    else {
-	$self->{default_data} = {} ;
     }
 
+    # store default data in a hash (more convenient)
+    $self->{default_data} = { map { $_ => 1 } @{$self->{default_list}} } ;
+
     if (defined $args{upstream_default_list}) {
 	$self->{upstream_default_list} = delete $args{upstream_default_list} ;
-	my %h = map { $_ => 1 } @{$self->{upstream_default_list}} ;
-	$self->{upstream_default_list} = \%h ;
-    }
-    else {
-	$self->{upstream_default_list} = {} ;
     }
 
+    # store upstream default data in a hash (more convenient)
+    $self->{upstream_default_data} = 
+      { map { $_ => 1 } @{$self->{upstream_default_list}} } ;
+
     Config::Model::Exception::Model
 	-> throw (
 		  object => $self,
@@ -323,7 +331,7 @@ sub setup_choice {
     my $self = shift ;
     my @choice = ref $_[0] ? @{$_[0]} : @_ ;
 
-    print "CheckList: setup_choice with @choice\n" if $::debug ;
+    $logger->debug("CheckList $self->{element_name}: setup_choice with @choice");
     # store all enum values in a hash. This way, checking
     # whether a value is present in the enum set is easier
     delete $self->{choice_hash} if defined $self->{choice_hash} ;
@@ -574,13 +582,14 @@ sub is_checked {
 	my $dat = $self->{data} ;
 	my $pre = $self->{preset} ;
 	my $def = $self->{default_data} ;
-	my $ud  = $self->{upstream_default_list} ;
+	my $ud  = $self->{upstream_default_data} ;
 	my $std_v = (defined $pre->{$choice} ? $pre->{$choice} : $def->{$choice}) || 0 ;
 
 	my $result 
 	  = $type eq 'custom'           ? ( $dat->{$choice} && ! $std_v ? 1 : 0 )
           : $type eq 'preset'           ? $pre->{$choice}
           : $type eq 'upstream_default' ? $ud ->{$choice}
+          : $type eq 'default'          ? $def->{$choice}
           : $type eq 'standard'         ? $std_v
           : defined $dat->{$choice}     ? $dat->{$choice}
           :                               $std_v ;
@@ -744,7 +753,7 @@ sub get_checked_list_as_hash {
     my $dat = $self->{data} ;
     my $pre = $self->{preset} ;
     my $def = $self->{default_data} ;
-    my $ud  = $self->{upstream_default_list} ;
+    my $ud  = $self->{upstream_default_data} ;
 
     # copy hash and return it
     my %std = (%h, %$def, %$pre ) ;
@@ -752,6 +761,7 @@ sub get_checked_list_as_hash {
       = $mode eq 'custom'   ? (%h, map { $dat->{$_} && ! $std{$_} ? ($_,1) : ()} keys %$dat )
       : $mode eq 'preset'   ? (%h, %$pre )
       : $mode eq 'upstream_default' ? %$ud
+      : $mode eq 'default'          ? %$def
       : $mode eq 'standard' ? %std
       :                       (%std, %$dat );
 
@@ -814,15 +824,20 @@ sub get {
     return $self->fetch(@_) ;
 }
 
-=head2 set( path , value )
+=head2 set( path , values )
 
-Set a value with a directory like path.
+Set a checklist with a directory like path. Since a checklist is a leaf, the path
+should be empty. The values are a comma separated list of checked items.
+
+Example : C<< $leaf->set('','A,C,Z') ; >>
 
 =cut
 
 sub set {
     my $self = shift ;
     my $path = shift ;
+    my $list = shift ;
+
     if ($path) {
 	Config::Model::Exception::User
 	    -> throw (
@@ -830,7 +845,8 @@ sub set {
 		      message => "set() called with a value with non-empty path: '$path'"
 		     ) ;
     }
-    return $self->set_checked_list(@_) ;
+
+    return $self->set_checked_list(split /,/,$list) ;
 }
 
 =head2 set_checked_list ( item1, item2, ..)
